@@ -1,7 +1,6 @@
 # (c) Copyright 2009-2012. CodeWeavers, Inc.
 
 import os.path
-import sys
 import time
 import traceback
 import xml.sax
@@ -9,7 +8,6 @@ import xml.sax.handler
 
 import cxproduct
 import cxlog
-import cxutils
 import c4profiles
 
 def install_sax_workaround():
@@ -38,14 +36,17 @@ class _C4PHandler(xml.sax.handler.ContentHandler):
     # install in the case of standalone c4p files.
     autorun = None
 
-    def __init__(self, trusted, source, origin):
+    def __init__(self, trusted, source, origin, disabled_licenses_only):
         xml.sax.handler.ContentHandler.__init__(self)
+        self.highlightedapps = []
         self.revokelist = []
         self.disabled_licenses = []
         self.apps = []
         self.parse_errors = []
         self.locator = None
 
+        self.disabled_licenses_only = disabled_licenses_only
+
         self._objects = [self]
         self._states = [_C4PHandler.state_init]
         self._skip = 0
@@ -456,9 +457,16 @@ class _C4PHandler(xml.sax.handler.ContentHandler):
     #####
 
     def _childof_c4p(self, name, attrs):
+        if self.disabled_licenses_only and name != 'disabledlicenses':
+            self._startof_skip(name)
+            return
+
         if name == 'revokelist':
             self._states.append(_C4PHandler.state_revokelist)
             cxlog.log_('c4parser', "objects=%d states=%d %s" % (len(self._objects), len(self._states), cxlog.to_str(name)))
+        elif name == 'highlightedapps':
+            self._states.append(_C4PHandler.state_highlightedapps)
+            cxlog.log_('c4parser', "objects=%d states=%d %s" % (len(self._objects), len(self._states), cxlog.to_str(name)))
         elif name == 'disabledlicenses':
             self._states.append(_C4PHandler.state_disabledlicenses)
             cxlog.log_('c4parser', "objects=%d states=%d %s" % (len(self._objects), len(self._states), cxlog.to_str(name)))
@@ -475,6 +483,34 @@ class _C4PHandler(xml.sax.handler.ContentHandler):
     state_c4p = (_childof_c4p, _end_simple, _unexpected_characters)
 
 
+    #####
+    #
+    # <highlightedapps> handler
+    #
+    #####
+
+    def _childof_highlightedapps(self, name, attrs):
+        if name == 'highlightedapp':
+            highlightedapp = c4profiles.C4HighlightedProfile()
+            highlightedapp.appid = attrs['appid']
+            highlightedapp.platform = attrs.get('platform', None)
+            self.highlightedapps.append(highlightedapp)
+            self._states.append(_C4PHandler.state_highlightedapp)
+        else:
+            self._startof_skip(name)
+
+    state_highlightedapps = (_childof_highlightedapps, _end_simple, _unexpected_characters)
+
+
+    #####
+    #
+    # <highlightedapp> handler
+    #
+    #####
+
+    state_highlightedapp = (_startof_unexpected, _end_simple, _unexpected_characters)
+
+
     #####
     #
     # <revokelist> handler
@@ -630,16 +666,14 @@ class _C4PHandler(xml.sax.handler.ContentHandler):
         c4profile = self._objects.pop()
         try:
             c4profile.validate()
-        except AttributeError:
-            exception = sys.exc_info()[1]
+        except AttributeError as exception:
             self.parse_errors.append(str(exception))
             cxlog.warn(cxlog.debug_str(exception))
             cxlog.warn("the %s profile contains errors, ignoring it" % cxlog.to_str(c4profile.appid))
             self.apps.pop()
             if cxlog.is_on('c4parser'):
                 traceback.print_exc()
-        except Exception: # pylint: disable=W0703
-            exception = sys.exc_info()[1]
+        except Exception as exception: # pylint: disable=W0703
             msg = "an unexpected error occurred while reading the %s profile, ignoring it" % c4profile.appid
             self.parse_errors.append(msg + "\n" + str(exception))
             cxlog.warn(msg)
@@ -716,9 +750,9 @@ class _C4PHandler(xml.sax.handler.ContentHandler):
         elif name == 'appbottlegroup':
             self._startof_string('application_group')
         elif name == 'downloadurl':
-            self._startof_stringmap('download_urls', 'lang', attrs)
+            self._startof_downloadmap('download_urls', attrs)
         elif name == 'downloadurl64':
-            self._startof_stringmap('download_urls_64bit', 'lang', attrs)
+            self._startof_downloadmap('download_urls_64bit', attrs)
         elif name == 'downloadpageurl':
             self._startof_stringmap('download_page_urls', 'lang', attrs)
         elif name == 'downloadglob':
@@ -732,6 +766,34 @@ class _C4PHandler(xml.sax.handler.ContentHandler):
 
     state_appprofile = (_childof_appprofile, _end_popobj, _unexpected_characters)
 
+    #####
+    #
+    # Download url handler
+    #
+    #####
+
+    def _startof_downloadmap(self, prop, attrs):
+        self._property = prop
+        self._key = attrs.get('lang', '')
+        self._checksum = attrs.get('checksum')
+        self._string = []
+        self._states.append(_C4PHandler.state_downloadmap)
+        cxlog.log_('c4parser', "objects=%d states=%d %s" % (len(self._objects), len(self._states), cxlog.to_str(prop)))
+
+    def _endof_downloadmap(self, _unused):
+        obj = self._objects[-1]
+        value = ''.join(self._string), self._checksum
+        cxlog.log_('c4parser', "obj=%s property=%s key=%s value=%s" % (cxlog.debug_str(obj), cxlog.to_str(self._property), cxlog.to_str(self._key), cxlog.debug_str(value)))
+        if self._property not in obj.__dict__:
+            obj.__dict__[self._property] = {self._key: value}
+        else:
+            obj.__dict__[self._property][self._key] = value
+        self._states.pop()
+        cxlog.log_('c4parser', "objects=%d states=%d %s" % (len(self._objects), len(self._states), cxlog.to_str(self._property)))
+
+    state_downloadmap = (_startof_unexpected, _endof_downloadmap, _string_characters)
+
+
     # <installedregistryglob> handler
 
     def _childof_installedregistryglob(self, name, _attrs):
@@ -847,10 +909,6 @@ class _C4PHandler(xml.sax.handler.ContentHandler):
                                      _C4PHandler.state_installregistry)
         elif name == 'prermfakedll':
             self._startof_stringlist('pre_rm_fake_dlls')
-        elif name == 'preinstallnsplugindll':
-            self._startof_stringlist('pre_install_nsplugin_dlls')
-        elif name == 'ignorensplugindll':
-            self._startof_stringlist('ignore_nsplugin_dlls')
         elif name == 'lnkfile':
             self._startof_objectlist('lnk_files',
                                      c4profiles.C4LnkFile(),
@@ -859,8 +917,6 @@ class _C4PHandler(xml.sax.handler.ContentHandler):
             self._startof_boolean('skip_menu_creation')
         elif name == 'skipassoc':
             self._startof_boolean('skip_assoc_creation')
-        elif name == 'installnsplugins':
-            self._startof_boolean('install_nsplugins')
         elif name == 'mainmenunever':
             self._startof_stringlist('mainmenu_never')
         elif name == 'defaulteassoc':
@@ -1035,12 +1091,15 @@ class _C4PHandler(xml.sax.handler.ContentHandler):
 #####
 
 
-class C4PFile(object):
+class C4PFile:
     """The in-memory representation of the contents of a c4p file."""
 
     def __init__(self, filename, trusted, source):
         self.mtime = 0
+        self.mtime_disabled_licenses = 0
+
         self._filename = filename
+        self._highlightedapps = None
         self._revokelist = None
         self._disabledlicenses = None
         self._profiles = None
@@ -1052,33 +1111,47 @@ class C4PFile(object):
             # A dropin file might not persist, so read it in right now!
             self.update()
 
-    def update(self):
+    def update(self, disabled_licenses_only=False):
         """Updates the C4PFile object if the file has changed since it was last
         read.
 
         Returns the mtime of file, or 0 if an error arises during parsing.
         """
 
-        if self._updatedonce and self._source == c4profiles.dropin:
-            # Dropin files aren't necessarily persistent, so we only
-            # read them the first time through.
-            return self.mtime
+        if self._source == c4profiles.dropin:
+            if self._updatedonce:
+                # Dropin files aren't necessarily persistent, so we only
+                # read them the first time through.
+                return self.mtime
+            # Dropin files are updated from the constructor, so this
+            # should never happen.
+            assert not disabled_licenses_only
+
         try:
             mtime = os.path.getmtime(self._filename)
         except OSError:
             # If the file is missing, or not accessible, then just store None
             # as its mtime
             mtime = None
-        if self.mtime == mtime:
-            return self.mtime
+
+        if self.mtime == mtime or (disabled_licenses_only and self.mtime_disabled_licenses == mtime):
+            return mtime
 
         self._updatedonce = True
+        self._parse_file(disabled_licenses_only)
+
+        self.mtime_disabled_licenses = mtime
+        if not disabled_licenses_only:
+            self.mtime = mtime
+        return mtime
+
+    def _parse_file(self, disabled_licenses_only):
         start = time.time()
         if self._trusted:
             origin = "CodeWeavers"
         else:
             origin = os.path.basename(self._filename)
-        handler = _C4PHandler(self._trusted, self._source, origin)
+        handler = _C4PHandler(self._trusted, self._source, origin, disabled_licenses_only)
 
         try:
             xml.sax.parse(self._filename, handler)
@@ -1087,17 +1160,15 @@ class C4PFile(object):
             handler.parse_errors.append(msg)
             cxlog.warn(msg)
             traceback.print_exc()
-            return 0
+            return
 
+        self._highlightedapps = handler.highlightedapps
         self._revokelist = handler.revokelist
         self._disabledlicenses = handler.disabled_licenses
         self._profiles = handler.apps
         self._autorun = handler.autorun
         cxlog.log("parsing '%s' took %0.3fs" % (cxlog.to_str(self._filename), time.time() - start))
 
-        self.mtime = mtime
-        return self.mtime
-
     def get_autorun_id(self):
         """Returns the application id of the profile that should be selected
         when this file is loaded, or None. If no autorun profile is specified
@@ -1179,7 +1250,7 @@ class C4PFile(object):
         supported_products = self.get_supported_product_ids()
 
         all_products = cxproduct.enum_products()
-        for product in cxutils.itervalues(all_products):
+        for product in all_products.values():
             if product['productid'] in supported_products or \
                product['productid'] == 'cxoffice' and self._version_number_to_tuple(product['productversion']) >= (11,) and 'cxgames' in supported_products:
                 result.append(product)
@@ -1239,6 +1310,15 @@ class C4PFile(object):
 
     malware_appid = property(_getmalware_appid)
 
+    def _gethighlightedapps(self):
+        """Returns a list of highlighted applications contained in the c4p
+        file.
+        """
+        self.update()
+        return self._highlightedapps
+
+    highlightedapps = property(_gethighlightedapps)
+
     def _getrevokelist(self):
         """Returns a list of the profile revocations contained in the c4p
         file.
@@ -1252,7 +1332,7 @@ class C4PFile(object):
         """Returns a list of the disabled license id's contained in the c4p
         file.
         """
-        self.update()
+        self.update(True)
         return self._disabledlicenses
 
     disabled_licenses = property(_getdisabledlicenses)
