@@ -17,6 +17,10 @@ import distversion
 # for localization
 from cxutils import cxgettext as _
 
+
+# The application id of the unknown application profile
+UNKNOWN = "com.codeweavers.unknown"
+
 # A mapping of the language code to the plain English name for use by the GUI.
 LANGUAGES = {
     '':      _('Default'),
@@ -77,7 +81,7 @@ LANGUAGES = {
     'mr':    _('Marathi'),
     'ms':    _('Malay (Malaysia)'),
     'mt':    _('Maltese'),
-    'nb':    _(u'Norwegian (Bokm\u00e5l)'),
+    'nb':    _('Norwegian (Bokm\u00e5l)'),
     'ne':    _('Nepali (India)'),
     'nl':    _('Dutch'),
     'no':    _('Norwegian'),
@@ -121,6 +125,15 @@ LANGUAGES = {
     'zu':    _('Zulu'),
     }
 
+RATING_DESCRIPTIONS = {
+    5: _("Runs Great"),
+    4: _("Runs Well"),
+    3: _("Limited Functionality"),
+    2: _("Installs, Will Not Run"),
+    1: _("Will Not Install"),
+    0: _("Untested"),
+    -1: _("Untrusted"),
+}
 
 # This is the set of environment variables which are allowed in expandable
 # profile strings. The values are given for documentation only.
@@ -147,124 +160,125 @@ ENVIRONMENT_VARIABLES = (
 
 # keep this in sync with installtask.InstallTask._component_categories
 _localized_categories_seq = {
-    u'Component': (_('Non-Applications'), _('Components')),
-    u'Component/Font': (_('Non-Applications'), _('Components'), _('Fonts')),
-    u'Component/Fonts': (_('Non-Applications'), _('Components'), _('Fonts')),
-    u'Component/Service Pack': (_('Non-Applications'), _('Components'), _('Service Packs')),
-    u'Educational Software & CBT': (_('Educational'),),
-    u'Educational': (_('Educational'),),
-    u'Games': (_('Games'),),
-    u'Games/1st Person Shooter': (_('Games'), _('First Person Shooter')),
-    u'Games/Action Games': (_('Games'), _('Action Games')),
-    u'Games/Adventures': (_('Games'), _('Adventure Games')),
-    u'Games/Adventure Games': (_('Games'), _('Adventure Games')),
-    u'Games/Adventures/Visual Novels': (_('Games'), _('Adventure Games'), _('Visual Novels')),
-    u'Games/Adventure Games/Visual Novels': (_('Games'), _('Adventure Games'), _('Visual Novels')),
-    u'Games/Casual Games': (_('Games'), _('Casual Games')),
-    u'Games/DOS Games': (_('Games'), _('DOS Games')),
-    u'Games/Educational games & children': (_('Games'), _('Educational Games')),
-    u'Games/Educational Games': (_('Games'), _('Educational Games')),
-    u'Games/Emulators': (_('Games'), _('Emulators')),
-    u'Games/Fighting Games': (_('Games'), _('Fighting Games')),
-    u'Games/First Person Shooter': (_('Games'), _('First Person Shooter')),
-    u'Games/Fun Stuff': (_('Games'), _('Fun Stuff')),
-    u'Games/Fun stuff': (_('Games'), _('Fun Stuff')),
-    u'Games/Game Tools': (_('Games'), _('Game Tools')),
-    u'Games/Online (MMORPG) Games': (_('Games'), _('Online Games')),
-    u'Games/Online Games': (_('Games'), _('Online Games')),
-    u'Games/Puzzle Games': (_('Games'), _('Puzzle Games')),
-    u'Games/Racing Games': (_('Games'), _('Racing Games')),
-    u'Games/Rhythm Games': (_('Games'), _('Rhythm Games')),
-    u'Games/Role Playing Games': (_('Games'), _('Role Playing Games')),
-    u'Games/Simulation Games': (_('Games'), _('Simulation Games')),
-    u'Games/Sports Games': (_('Games'), _('Sports Games')),
-    u'Games/Survival Games': (_('Games'), _('Survival Games')),
-    u'Games/Strategy Games': (_('Games'), _('Strategy Games')),
-    u'Multimedia': (_('Multimedia'),),
-    u'Multimedia/Audio': (_('Multimedia'), _('Audio')),
-    u'Multimedia/Audio/Audio Players': (_('Multimedia'), _('Audio'), _('Audio Players')),
-    u'Multimedia/Audio/Music Composition': (_('Multimedia'), _('Audio'), _('Music Composition')),
-    u'Multimedia/Audio/Sound Editing': (_('Multimedia'), _('Audio'), _('Sound Editing')),
-    u'Multimedia/Graphics': (_('Multimedia'), _('Graphics')),
-    u'Multimedia/Graphics/Animation, Rendering & 3D': (_('Multimedia'), _('Graphics'), _('Animation, Rendering & 3D')),
-    u'Multimedia/Graphics/Graphics Editing': (_('Multimedia'), _('Graphics'), _('Photo Editing')),
-    u'Multimedia/Graphics/Graphics Viewer': (_('Multimedia'), _('Graphics'), _('Graphics Viewer')),
-    u'Multimedia/Graphics/Photo Editing': (_('Multimedia'), _('Graphics'), _('Photo Editing')),
-    u'Multimedia/Media Players': (_('Multimedia'), _('Media Players')),
-    u'Multimedia/Music Composition': (_('Multimedia'), _('Music Composition')),
-    u'Multimedia/Photo Editing': (_('Multimedia'), _('Photo Editing')),
-    u'Multimedia/Video': (_('Multimedia'), _('Video')),
-    u'Multimedia/Video/Video Editing': (_('Multimedia'), _('Video'), _('Video Editing')),
-    u'Networking & Communication': (_('Networking & Communication'),),
-    u'Networking & Communication/Browsers': (_('Networking & Communication'), _('Browsers')),
-    u'Networking & Communication/Chat, Instant Messaging & Telephony': (_('Networking & Communication'), _('Chat & Instant Messaging')),
-    u'Networking & Communication/Chat & Instant Messaging': (_('Networking & Communication'), _('Chat & Instant Messaging')),
-    u'Networking & Communication/Email, News & Groupware': (_('Networking & Communication'), _('Email & News Readers')),
-    u'Networking & Communication/Email & News': (_('Networking & Communication'), _('Email & News Readers')),
-    u'Networking & Communication/Email & News Readers': (_('Networking & Communication'), _('Email & News Readers')),
-    u'Networking & Communication/File transfer & Sharing': (_('Networking & Communication'), _('File Transfer & Sharing')),
-    u'Networking & Communication/File Transfer & Sharing': (_('Networking & Communication'), _('File Transfer & Sharing')),
-    u'Networking & Communication/Net Tools': (_('Networking & Communication'), _('Net Tools')),
-    u'Networking & Communication/Remote Access': (_('Networking & Communication'), _('Remote Access')),
-    u'Non Applications': (_('Non-Applications'),),
-    u'Non Applications/CrossTie Snippets': (_('Non-Applications'), _('CrossTie Snippets')),
-    u'Non-Applications': (_('Non-Applications'),),
-    u'Non-Applications/CrossTie Snippets': (_('Non-Applications'), _('CrossTie Snippets')),
-    u'Non-Applications/Components': (_('Non-Applications'), _('Components')),
-    u'Non-Applications/Components/Fonts': (_('Non-Applications'), _('Components'), _('Fonts')),
-    u'Non-Applications/Components/Service Packs': (_('Non-Applications'), _('Components'), _('Service Packs')),
-    u'Productivity': (_('Productivity'),),
-    u'Productivity/Database': (_('Productivity'), _('Database')),
-    u'Productivity/Defect Tracking & Help Desk': (_('Productivity'), _('Project Tracking & Help Desk')),
-    u'Productivity/Desktop Publishing': (_('Productivity'), _('Desktop Publishing')),
-    u'Productivity/Finance, Accounting & Project': (_('Productivity'), _('Finance & Accounting')),
-    u'Productivity/Finance & Accounting': (_('Productivity'), _('Finance & Accounting')),
-    u'Productivity/Project Tracking & Help Desk': (_('Productivity'), _('Project Tracking & Help Desk')),
-    u'Productivity/Legal & law': (_('Productivity'), _('Legal & Law')),
-    u'Productivity/Legal & Law': (_('Productivity'), _('Legal & Law')),
-    u'Productivity/Office Suites': (_('Productivity'), _('Office Suites')),
-    u'Productivity/Office Utilities': (_('Productivity'), _('Office Utilities')),
-    u'Productivity/Presentation': (_('Productivity'), _('Spreadsheet & Presentation')),
-    u'Productivity/Spreadsheet & Presentation': (_('Productivity'), _('Spreadsheet & Presentation')),
-    u'Productivity/Spreadsheet': (_('Productivity'), _('Spreadsheet & Presentation')),
-    u'Productivity/Text Editors': (_('Productivity'), _('Text Editors & Document Viewers')),
-    u'Productivity/Text Editors & Document Viewers': (_('Productivity'), _('Text Editors & Document Viewers')),
-    u'Productivity/Web Design': (_('Productivity'), _('Web Design')),
-    u'Productivity/Word Processing': (_('Productivity'), _('Word Processing')),
-    u'Programming & Software Engineering': (_('Programming & Development Tools'),),
-    u'Programming & Development Tools': (_('Programming & Development Tools'),),
-    u'Reference, Documentation & Info': (_('Reference, Documentation & Informational'),),
-    u'Reference, Documentation & Informational': (_('Reference, Documentation & Informational'),),
-    u'Reference, Documentation & Informational/Cooking & Nutrition': (_('Reference, Documentation & Informational'), _('Cooking & Nutrition')),
-    u'Reference, Documentation & Informational/Religious Studies': (_('Reference, Documentation & Informational'), _('Religious Studies')),
-    u'Reference, Documentation & Informational/Foreign Language & Speech': (_('Reference, Documentation & Informational'), _('Foreign Language & Speech')),
-    u'Reference, Documentation & Informational/Genealogy': (_('Reference, Documentation & Informational'), _('Genealogy')),
-    u'Scientific, Technical & Math': (_('Scientific, Technical & Math'),),
-    u'Scientific, Technical & Math/Astronomy': (_('Scientific, Technical & Math'), _('Astronomy & Weather')),
-    u'Scientific, Technical & Math/Astronomy & Weather': (_('Scientific, Technical & Math'), _('Astronomy & Weather')),
-    u'Scientific, Technical & Math/Biology': (_('Scientific, Technical & Math'), _('Biology & Chemistry')),
-    u'Scientific, Technical & Math/Biology & Chemistry': (_('Scientific, Technical & Math'), _('Biology & Chemistry')),
-    u'Scientific, Technical & Math/CAD & CAE': (_('Scientific, Technical & Math'), _('CAD & EDA')),
-    u'Scientific, Technical & Math/CAD & EDA': (_('Scientific, Technical & Math'), _('CAD & EDA')),
-    u'Scientific, Technical & Math/Chemistry': (_('Scientific, Technical & Math'), _('Biology & Chemistry')),
-    u'Scientific, Technical & Math/EDA': (_('Scientific, Technical & Math'), _('CAD & EDA')),
-    u'Scientific, Technical & Math/Flowchart, Diagramming & Graphs': (_('Scientific, Technical & Math'), _('Flowchart, Diagramming & Graphs')),
-    u'Scientific, Technical & Math/Mapping & GPS': (_('Scientific, Technical & Math'), _('Mapping & GPS')),
-    u'Scientific, Technical & Math/Mathematics': (_('Scientific, Technical & Math'), _('Mathematics')),
-    u'Special Purpose': (_('Special Purpose'),),
-    u'Special Purpose/Crafting': (_('Special Purpose'), _('Crafting')),
-    u'Special Purpose/Health & Medical': (_('Special Purpose'), _('Health & Medical')),
-    u'Special Purpose/Device-Specific Interfaces': (_('Special Purpose'), _('Device-Specific Interfaces')),
-    u'Unknown': (_('Unknown'),),
-    u'Utilities': (_('Utilities'),),
-    u'Utilities/Compression': (_('Utilities'), _('Compression & Conversion')),
-    u'Utilities/Compression & Conversion': (_('Utilities'), _('Compression & Conversion')),
-    u'Utilities/File System': (_('Utilities'), _('File System')),
-    u'Utilities/Catalog & Inventory': (_('Utilities'), _('Catalog & Inventory')),
-    u'Utilities/Disc Burning': (_('Utilities'), _('Disc Burning')),
+    'Component': (_('Non-Applications'), _('Components')),
+    'Component/Font': (_('Non-Applications'), _('Components'), _('Fonts')),
+    'Component/Fonts': (_('Non-Applications'), _('Components'), _('Fonts')),
+    'Component/Service Pack': (_('Non-Applications'), _('Components'), _('Service Packs')),
+    'Educational Software & CBT': (_('Educational'),),
+    'Educational': (_('Educational'),),
+    'Games': (_('Games'),),
+    'Games/1st Person Shooter': (_('Games'), _('First Person Shooter')),
+    'Games/Action Games': (_('Games'), _('Action Games')),
+    'Games/Adventures': (_('Games'), _('Adventure Games')),
+    'Games/Adventure Games': (_('Games'), _('Adventure Games')),
+    'Games/Adventures/Visual Novels': (_('Games'), _('Adventure Games'), _('Visual Novels')),
+    'Games/Adventure Games/Visual Novels': (_('Games'), _('Adventure Games'), _('Visual Novels')),
+    'Games/Casual Games': (_('Games'), _('Casual Games')),
+    'Games/DOS Games': (_('Games'), _('DOS Games')),
+    'Games/Educational games & children': (_('Games'), _('Educational Games')),
+    'Games/Educational Games': (_('Games'), _('Educational Games')),
+    'Games/Emulators': (_('Games'), _('Emulators')),
+    'Games/Fighting Games': (_('Games'), _('Fighting Games')),
+    'Games/First Person Shooter': (_('Games'), _('First Person Shooter')),
+    'Games/Fun Stuff': (_('Games'), _('Fun Stuff')),
+    'Games/Fun stuff': (_('Games'), _('Fun Stuff')),
+    'Games/Game Tools': (_('Games'), _('Game Tools')),
+    'Games/Online (MMORPG) Games': (_('Games'), _('Online Games')),
+    'Games/Online Games': (_('Games'), _('Online Games')),
+    'Games/Puzzle Games': (_('Games'), _('Puzzle Games')),
+    'Games/Racing Games': (_('Games'), _('Racing Games')),
+    'Games/Rhythm Games': (_('Games'), _('Rhythm Games')),
+    'Games/Role Playing Games': (_('Games'), _('Role Playing Games')),
+    'Games/Simulation Games': (_('Games'), _('Simulation Games')),
+    'Games/Sports Games': (_('Games'), _('Sports Games')),
+    'Games/Survival Games': (_('Games'), _('Survival Games')),
+    'Games/Strategy Games': (_('Games'), _('Strategy Games')),
+    'Multimedia': (_('Multimedia'),),
+    'Multimedia/Audio': (_('Multimedia'), _('Audio')),
+    'Multimedia/Audio/Audio Players': (_('Multimedia'), _('Audio'), _('Audio Players')),
+    'Multimedia/Audio/Music Composition': (_('Multimedia'), _('Audio'), _('Music Composition')),
+    'Multimedia/Audio/Sound Editing': (_('Multimedia'), _('Audio'), _('Sound Editing')),
+    'Multimedia/Graphics': (_('Multimedia'), _('Graphics')),
+    'Multimedia/Graphics/Animation, Rendering & 3D': (_('Multimedia'), _('Graphics'), _('Animation, Rendering & 3D')),
+    'Multimedia/Graphics/Graphics Editing': (_('Multimedia'), _('Graphics'), _('Photo Editing')),
+    'Multimedia/Graphics/Graphics Viewer': (_('Multimedia'), _('Graphics'), _('Graphics Viewer')),
+    'Multimedia/Graphics/Photo Editing': (_('Multimedia'), _('Graphics'), _('Photo Editing')),
+    'Multimedia/Media Players': (_('Multimedia'), _('Media Players')),
+    'Multimedia/Music Composition': (_('Multimedia'), _('Music Composition')),
+    'Multimedia/Photo Editing': (_('Multimedia'), _('Photo Editing')),
+    'Multimedia/Video': (_('Multimedia'), _('Video')),
+    'Multimedia/Video/Video Editing': (_('Multimedia'), _('Video'), _('Video Editing')),
+    'Multimedia/Video Editing': (_('Multimedia'), _('Video Editing')),
+    'Networking & Communication': (_('Networking & Communication'),),
+    'Networking & Communication/Browsers': (_('Networking & Communication'), _('Browsers')),
+    'Networking & Communication/Chat, Instant Messaging & Telephony': (_('Networking & Communication'), _('Chat & Instant Messaging')),
+    'Networking & Communication/Chat & Instant Messaging': (_('Networking & Communication'), _('Chat & Instant Messaging')),
+    'Networking & Communication/Email, News & Groupware': (_('Networking & Communication'), _('Email & News Readers')),
+    'Networking & Communication/Email & News': (_('Networking & Communication'), _('Email & News Readers')),
+    'Networking & Communication/Email & News Readers': (_('Networking & Communication'), _('Email & News Readers')),
+    'Networking & Communication/File transfer & Sharing': (_('Networking & Communication'), _('File Transfer & Sharing')),
+    'Networking & Communication/File Transfer & Sharing': (_('Networking & Communication'), _('File Transfer & Sharing')),
+    'Networking & Communication/Net Tools': (_('Networking & Communication'), _('Net Tools')),
+    'Networking & Communication/Remote Access': (_('Networking & Communication'), _('Remote Access')),
+    'Non Applications': (_('Non-Applications'),),
+    'Non Applications/CrossTie Snippets': (_('Non-Applications'), _('CrossTie Snippets')),
+    'Non-Applications': (_('Non-Applications'),),
+    'Non-Applications/CrossTie Snippets': (_('Non-Applications'), _('CrossTie Snippets')),
+    'Non-Applications/Components': (_('Non-Applications'), _('Components')),
+    'Non-Applications/Components/Fonts': (_('Non-Applications'), _('Components'), _('Fonts')),
+    'Non-Applications/Components/Service Packs': (_('Non-Applications'), _('Components'), _('Service Packs')),
+    'Productivity': (_('Productivity'),),
+    'Productivity/Database': (_('Productivity'), _('Database')),
+    'Productivity/Defect Tracking & Help Desk': (_('Productivity'), _('Project Tracking & Help Desk')),
+    'Productivity/Desktop Publishing': (_('Productivity'), _('Desktop Publishing')),
+    'Productivity/Finance, Accounting & Project': (_('Productivity'), _('Finance & Accounting')),
+    'Productivity/Finance & Accounting': (_('Productivity'), _('Finance & Accounting')),
+    'Productivity/Project Tracking & Help Desk': (_('Productivity'), _('Project Tracking & Help Desk')),
+    'Productivity/Legal & law': (_('Productivity'), _('Legal & Law')),
+    'Productivity/Legal & Law': (_('Productivity'), _('Legal & Law')),
+    'Productivity/Office Suites': (_('Productivity'), _('Office Suites')),
+    'Productivity/Office Utilities': (_('Productivity'), _('Office Utilities')),
+    'Productivity/Presentation': (_('Productivity'), _('Spreadsheet & Presentation')),
+    'Productivity/Spreadsheet & Presentation': (_('Productivity'), _('Spreadsheet & Presentation')),
+    'Productivity/Spreadsheet': (_('Productivity'), _('Spreadsheet & Presentation')),
+    'Productivity/Text Editors': (_('Productivity'), _('Text Editors & Document Viewers')),
+    'Productivity/Text Editors & Document Viewers': (_('Productivity'), _('Text Editors & Document Viewers')),
+    'Productivity/Web Design': (_('Productivity'), _('Web Design')),
+    'Productivity/Word Processing': (_('Productivity'), _('Word Processing')),
+    'Programming & Software Engineering': (_('Programming & Development Tools'),),
+    'Programming & Development Tools': (_('Programming & Development Tools'),),
+    'Reference, Documentation & Info': (_('Reference, Documentation & Informational'),),
+    'Reference, Documentation & Informational': (_('Reference, Documentation & Informational'),),
+    'Reference, Documentation & Informational/Cooking & Nutrition': (_('Reference, Documentation & Informational'), _('Cooking & Nutrition')),
+    'Reference, Documentation & Informational/Religious Studies': (_('Reference, Documentation & Informational'), _('Religious Studies')),
+    'Reference, Documentation & Informational/Foreign Language & Speech': (_('Reference, Documentation & Informational'), _('Foreign Language & Speech')),
+    'Reference, Documentation & Informational/Genealogy': (_('Reference, Documentation & Informational'), _('Genealogy')),
+    'Scientific, Technical & Math': (_('Scientific, Technical & Math'),),
+    'Scientific, Technical & Math/Astronomy': (_('Scientific, Technical & Math'), _('Astronomy & Weather')),
+    'Scientific, Technical & Math/Astronomy & Weather': (_('Scientific, Technical & Math'), _('Astronomy & Weather')),
+    'Scientific, Technical & Math/Biology': (_('Scientific, Technical & Math'), _('Biology & Chemistry')),
+    'Scientific, Technical & Math/Biology & Chemistry': (_('Scientific, Technical & Math'), _('Biology & Chemistry')),
+    'Scientific, Technical & Math/CAD & CAE': (_('Scientific, Technical & Math'), _('CAD & EDA')),
+    'Scientific, Technical & Math/CAD & EDA': (_('Scientific, Technical & Math'), _('CAD & EDA')),
+    'Scientific, Technical & Math/Chemistry': (_('Scientific, Technical & Math'), _('Biology & Chemistry')),
+    'Scientific, Technical & Math/EDA': (_('Scientific, Technical & Math'), _('CAD & EDA')),
+    'Scientific, Technical & Math/Flowchart, Diagramming & Graphs': (_('Scientific, Technical & Math'), _('Flowchart, Diagramming & Graphs')),
+    'Scientific, Technical & Math/Mapping & GPS': (_('Scientific, Technical & Math'), _('Mapping & GPS')),
+    'Scientific, Technical & Math/Mathematics': (_('Scientific, Technical & Math'), _('Mathematics')),
+    'Special Purpose': (_('Special Purpose'),),
+    'Special Purpose/Crafting': (_('Special Purpose'), _('Crafting')),
+    'Special Purpose/Health & Medical': (_('Special Purpose'), _('Health & Medical')),
+    'Special Purpose/Device-Specific Interfaces': (_('Special Purpose'), _('Device-Specific Interfaces')),
+    'Unknown': (_('Unknown'),),
+    'Utilities': (_('Utilities'),),
+    'Utilities/Compression': (_('Utilities'), _('Compression & Conversion')),
+    'Utilities/Compression & Conversion': (_('Utilities'), _('Compression & Conversion')),
+    'Utilities/File System': (_('Utilities'), _('File System')),
+    'Utilities/Catalog & Inventory': (_('Utilities'), _('Catalog & Inventory')),
+    'Utilities/Disc Burning': (_('Utilities'), _('Disc Burning')),
     }
 
-_localized_categories = dict((k, '/'.join(v)) for (k, v) in cxutils.iteritems(_localized_categories_seq))
+_localized_categories = dict((k, '/'.join(v)) for (k, v) in _localized_categories_seq.items())
 
 #####
 #
@@ -306,11 +320,11 @@ def _validate_regular_expression(pattern, location, allow_empty=True):
             if re.match(pattern, '') and not allow_empty:
                 raise AttributeError("the %s regular expression for %s is overly broad" % (pattern, location))
         except re.error:
-            raise AttributeError("%s is an invalid regular expression for %s" % (pattern, location))
+            raise AttributeError("%s is an invalid regular expression for %s" % (pattern, location)) #pylint: disable=W0707
 
 def _validate_language_field(field):
     # return True if there is at least one non-empty value
-    for value in cxutils.itervalues(field):
+    for value in field.values():
         if value:
             return True
     return False
@@ -349,7 +363,7 @@ def _dump_fields(out, indent, obj, fields):
             continue
         value = obj.__dict__[field]
         if isinstance(value, dict):
-            for key in sorted(cxutils.keys(value)):
+            for key in sorted(value):
                 out.write("%s%s{%s} =" % (indent, field, key))
                 _dump_value(out, indent, value[key])
         elif isinstance(value, list):
@@ -380,7 +394,7 @@ def _copy_value(value):
         return copy_func()
 
 def _copy_values(attributes):
-    for key, value in cxutils.iteritems(attributes):
+    for key, value in attributes.items():
         if key != 'parent':
             attributes[key] = _copy_value(value)
 
@@ -400,7 +414,7 @@ def _update_value(this, oth):
         return result
 
 def _update_values(attributes, others, exclude=(), special=()):
-    for key, value in cxutils.iteritems(others):
+    for key, value in others.items():
         if key in exclude:
             continue
         if key in attributes:
@@ -418,7 +432,7 @@ def _update_html(this, oth):
     return '%s<br/>%s' % (this, oth)
 
 def _update_html_dict(this, oth):
-    _update_values(this, oth, special={cxutils.unicode_type: _update_html})
+    _update_values(this, oth, special={str: _update_html})
     return this
 
 
@@ -428,7 +442,7 @@ def _update_html_dict(this, oth):
 #
 #####
 
-class C4Condition(object):
+class C4Condition:
     """Represents a boolean expression."""
 
     @cxdecorators.abstractmethod
@@ -601,7 +615,7 @@ class C4ConditionMatch(C4Condition):
 #
 #####
 
-class C4CDGlob(object):
+class C4CDGlob:
     """Contains a specific glob for detecting an application CD."""
 
     ### Property defaults
@@ -630,7 +644,7 @@ class C4CDGlob(object):
         _dump_fields(out, indent, self, ('glob', 'patterns'))
 
 
-class C4CDProfile(object):
+class C4CDProfile:
     """Contains the C4 CD Profile information."""
 
     ### Property defaults
@@ -680,7 +694,7 @@ class C4CDProfile(object):
 #####
 
 
-class C4RegistryGlob(object):
+class C4RegistryGlob:
     """Contains a specific registry glob for detecting an installed
     application.
     """
@@ -720,7 +734,7 @@ class C4RegistryGlob(object):
                      ('key_glob', 'value_pattern', 'data_pattern'))
 
 
-class C4ApplicationProfile(object):
+class C4ApplicationProfile:
     """Contains the C4 Application Profile information."""
 
     ### Enumerated types
@@ -893,6 +907,20 @@ class C4ApplicationProfile(object):
 
     description = property(_getdescription)
 
+    def _get_download_page_url(self):
+        """Returns the download page url for the current language.
+
+        If no url is available for the current language, then the
+        default url is returned.
+        """
+        if not self.download_page_urls:
+            return None
+
+        _lang, url = cxutils.get_language_value(self.download_page_urls)
+        return url
+
+    download_page_url = property(_get_download_page_url)
+
 
     def _get_platform(self):
         """Ratings for Chrome OS are stored as "Android" for historical reasons."""
@@ -997,6 +1025,11 @@ class C4ApplicationProfile(object):
 
     medal_count = property(_getmedal_count)
 
+    def _get_rating_description(self):
+        return RATING_DESCRIPTIONS[self.medal_rating]
+
+    rating_description = property(_get_rating_description)
+
     def _getis_supported(self):
         """Returns true if the application is supported."""
         try:
@@ -1046,7 +1079,7 @@ class C4ApplicationProfile(object):
                           '.otf', '.rar', '.ttc', '.ttf', '.tgz', '.zip'))
         globs = set(('', 'install.exe', 'install.msi', 'installer.exe',
                      'installer.msi', 'setup.exe', 'setup.msi'))
-        for _locale, url in cxutils.iteritems(self.download_urls):
+        for _locale, (url, _checksum) in self.download_urls.items():
             glob = cxurlget.url_to_basename(url)
             if not cxutils.is_valid_windows_filename(glob):
                 continue
@@ -1107,7 +1140,7 @@ class C4ApplicationProfile(object):
 
 UNIX_ENVIRONMENT_BLACKLIST = frozenset(('IFS', 'LD_LIBRARY_PATH', 'PATH'))
 
-class C4EnvVar(object):
+class C4EnvVar:
     """Describes an environment variable to set."""
 
     ### Property defaults
@@ -1137,7 +1170,7 @@ class C4EnvVar(object):
         _dump_fields(out, indent, self, ('name', 'value'))
 
 
-class C4FilesToCopy(object):
+class C4FilesToCopy:
     """Specifies a set of files to copy."""
 
     ### Property defaults
@@ -1167,7 +1200,7 @@ class C4FilesToCopy(object):
         _dump_fields(out, indent, self, ('glob', 'destination'))
 
 
-class C4RegistryValue(object):
+class C4RegistryValue:
     """Specifies a registry value to set."""
 
     ### Property defaults
@@ -1195,7 +1228,7 @@ class C4RegistryValue(object):
         _dump_fields(out, indent, self, ('name', 'data'))
 
 
-class C4Registry(object):
+class C4Registry:
     """Specifies a set of registry values to set."""
 
     ### Property defaults
@@ -1210,19 +1243,19 @@ class C4Registry(object):
 
     ### Instance validation and dumping
 
-    ROOT_KEYS = frozenset((u'HKEY_CLASSES_ROOT', u'HKCR',
-                           u'HKEY_CURRENT_USER', u'HKCU',
-                           u'HKEY_LOCAL_MACHINE', u'HKLM',
-                           u'HKEY_USERS', u'HKU'))
+    ROOT_KEYS = frozenset(('HKEY_CLASSES_ROOT', 'HKCR',
+                           'HKEY_CURRENT_USER', 'HKCU',
+                           'HKEY_LOCAL_MACHINE', 'HKLM',
+                           'HKEY_USERS', 'HKU'))
 
     def validate(self, app_name):
         """Checks that all the mandatory fields have been set."""
         if not self.key:
             raise AttributeError("C4Registry.key is not set for application %s" % app_name)
         # pylint: disable=E1135
-        if u'\\' not in self.key:
+        if '\\' not in self.key:
             raise AttributeError("C4Registry.key %s contains no backslashes for application %s" % (self.key, app_name))
-        if self.key.split(u'\\', 1)[0] not in self.ROOT_KEYS:
+        if self.key.split('\\', 1)[0] not in self.ROOT_KEYS:
             raise AttributeError("C4Registry.key %s has an invalid root key for application %s" % (self.key, app_name))
         if self.values:
             for value in self.values:
@@ -1233,7 +1266,7 @@ class C4Registry(object):
         _dump_fields(out, indent, self, ('key', 'values'))
 
 
-class C4LnkFile(object):
+class C4LnkFile:
     """Specifies a wnidows shortcut to create."""
 
     ### Property defaults
@@ -1265,7 +1298,7 @@ class C4LnkFile(object):
         _dump_fields(out, indent, self, ('shortcut', 'target', 'workdir'))
 
 
-class C4InstallerProfile(object):
+class C4InstallerProfile:
     """Contains the C4 Installer Profile information."""
 
     ### 'Pre-Install' Property defaults
@@ -1369,12 +1402,6 @@ class C4InstallerProfile(object):
     # A list of fake dlls to remove before running the installer.
     pre_rm_fake_dlls = tuple()
 
-    # A list of Netscape plugin dlls to export before running the installer.
-    pre_install_nsplugin_dlls = tuple()
-
-    # A list of Netscape plugins to ignore.
-    ignore_nsplugin_dlls = tuple()
-
     # A list of C4LnkFile objects describing the Windows shortcuts to be
     # created.
     lnk_files = tuple()
@@ -1387,9 +1414,6 @@ class C4InstallerProfile(object):
     # Set to True if the application is known to not create any association.
     skip_assoc_creation = False
 
-    # Set to True if the installer should export browser plugins to the host.
-    install_nsplugins = False
-
     # The list of EAssocs that should be made the default.
     default_eassocs = tuple()
 
@@ -1483,9 +1507,8 @@ class C4InstallerProfile(object):
                       'selfextract_silent_options', 'zip_encoding',
                       'files_to_copy', 'post_install_reboot',
                       'pre_install_registry', 'pre_rm_fake_dlls',
-                      'pre_install_nsplugin_dlls', 'ignore_nsplugin_dlls',
                       'lnk_files', 'skip_menu_creation', 'skip_assoc_creation',
-                      'install_nsplugins', 'default_eassocs', 'alt_eassocs',
+                      'default_eassocs', 'alt_eassocs',
                       'post_registerdll', 'post_install_urls'))
 
 
@@ -1547,7 +1570,7 @@ class C4InstallerProfile(object):
 #
 #####
 
-class C4CXVersion(object):
+class C4CXVersion:
     """Identifies a product plus version combination which is compatible with
     the corresponding profile.
     """
@@ -1595,7 +1618,7 @@ class C4CXVersion(object):
         _update_values(self.__dict__, oth.__dict__)
 
 
-class C4PTimestamp(object):
+class C4PTimestamp:
     datetime = None
 
     def __init__(self, timestamp):
@@ -1639,7 +1662,7 @@ class C4PTimestamp(object):
     def __ge__(self, oth):
         return self.__cmp__(oth) >= 0
 
-class C4CXContributor(object):
+class C4CXContributor:
     """Identifies a contributor to the profile
     """
 
@@ -1693,7 +1716,7 @@ _medal_ratings = {
     'untrusted': -1,
     }
 
-class C4CXMedal(object):
+class C4CXMedal:
     """Identifies a contributor to the profile
     """
 
@@ -1765,7 +1788,7 @@ class C4CXMedal(object):
 
 
 builtin, download, dropin = range(3)
-class C4Profile(object):
+class C4Profile:
     """This is an umbrella class which ties a C4 entry's CD, Application and
     Installer profiles together.
 
@@ -1797,6 +1820,10 @@ class C4Profile(object):
     # This field is mandatory and thus defaults to None.
     appid = None
 
+    # A boolean that is true if the app should appear in the highlighted
+    # applications section of the package view.
+    is_highlighted = False
+
     # The timestamp of the profile's last modification.
     # This field is optional and defaults to None.
     timestamp = None
@@ -2021,8 +2048,8 @@ class C4Profile(object):
         # For now, just treat com.codeweavers.c4.NNN as special.
         # In the future, we may want to add a field in c4p files for this.
         # pylint: disable=E1101
-        if self.appid.startswith('com.codeweavers.c4.'):
-            c4_id = self.appid[19:]
+        if self.appid and self.appid.startswith('com.codeweavers.c4.'):
+            c4_id = self.appid.replace('com.codeweavers.c4.', '')
             if c4_id.isdigit():
                 return 'http://www.codeweavers.com/compatibility/browse/name?app_id=%s' % c4_id
         return ''
@@ -2045,13 +2072,11 @@ class C4Profile(object):
 
     is_component = property(_is_component)
 
+    def _is_unknown(self):
+        """Returns True if this is the unknown profile."""
+        return self.appid == UNKNOWN
 
-    def _is_nsplugin(self):
-        """Returns True if the nsplugin flag is set."""
-        # pylint: disable=E1101
-        return 'nsplugin' in self.app_profile.flags
-
-    is_nsplugin = property(_is_nsplugin)
+    is_unknown = property(_is_unknown)
 
     def _get_contributors(self):
         contributors = ""
@@ -2079,12 +2104,47 @@ class C4Profile(object):
     contributors_html = property(_get_contributors_html)
 
 
+#####
+#
+# C4HighlightedProfile class
+#
+#####
+class C4HighlightedProfile:
+    """Contains the identifying information of the C4Profile instances to
+    revoke.
+    """
+
+    ### Property defaults
+
+    # These class variables provide defaults in case the corresponding
+    # instance variable is not set. They should all be immutable objects,
+
+    # A string storing the application id, usually a C4 identifier.
+    # This field is mandatory and thus defaults to None.
+    appid = None
+
+    # The platform the app is highlighted on
+    platform = None
+
+
+    ### Instance validation and dumping
+
+    def validate(self):
+        """Checks that all the mandatory fields have been set on this profile.
+        """
+        if not self.appid or not self.platform:
+            raise AttributeError("C4HighlightedProfile.appid is not set")
+
+    def dump(self, out, indent=""):
+        _dump_fields(out, indent, self, ('appid', 'platform'))
+
+
 #####
 #
 # C4RevokeProfile class
 #
 #####
-class C4RevokeProfile(object):
+class C4RevokeProfile:
     """Contains the identifying information of the C4Profile instances to
     revoke.
     """
@@ -2125,7 +2185,7 @@ class C4RevokeProfile(object):
 # supported_list is a simple list of app ids. If supported_list
 #  is non-NULL, then /only/ apps in the list should appear
 #  with a 'supported' medal in the GUI.
-class supported_list(object):
+class supported_list:
     _supported_list_read = False
     _supported_list = []
 
@@ -2141,7 +2201,7 @@ class supported_list(object):
         filename = os.path.join(cxutils.CX_ROOT, "share/crossover/data", ".supported")
 
         try:
-            f = cxutils.open_text(filename)
+            f = open(filename, 'rt', encoding='ascii') # pylint: disable=R1732
         except IOError:
             # No such file
             return
