@@ -24,7 +24,7 @@ STATUS_STR = ["queued", "runnable", "running", "done"]
 _TASK_SCHEDULED = set((TASK_RUNNING, TASK_DONE))
 
 
-class AIETask(object):
+class AIETask:
     """Represents a task that can be run by the AIEScheduler."""
 
     # If an error occurs, set this attribute to the appropriate error message.
@@ -36,6 +36,9 @@ class AIETask(object):
     # Set to true if the error message contains Pango markup
     ismarkuperror = False
 
+    # Set to true if the task requires user interaction
+    interactive = False
+
     #####
     #
     # Task configuration
@@ -192,7 +195,9 @@ class AIETask(object):
                     if parent.status == TASK_DONE:
                         # The parent was already marked as done, so its
                         # parents may need to be made runnable
-                        unblocked.append(parent)
+                        # Modifying the list while iterating it is intentional,
+                        # and parent will be examined in this loop.
+                        unblocked.append(parent) # pylint: disable=W4701
                     else:
                         self.scheduler.add_runnable(parent)
                 task.parents.clear()
@@ -350,7 +355,7 @@ class AIEScheduler(cxobjc.Proxy):
 
     def lock(self):
         """Locks the scheduler to make some operations thread-safe."""
-        self._lock.acquire()
+        self._lock.acquire() # pylint: disable=R1732
 
 
     def unlock(self):
