@@ -156,10 +156,8 @@ profile graph into an installation tasks graph:
 
 import os.path
 import os
-import shutil
 import time
 
-import cxproduct
 import cxlog
 import cxutils
 
@@ -190,7 +188,7 @@ class Engine(cxaiebase.AIEScheduler):
     #####
 
     @cxobjc.python_method
-    def start_logging(self, logfile, channels):
+    def start_logging(self, logfile, channels, extra_env):
         """Initializes the global environment block and sets up logging.
 
         Setting up logging involves starting cxlogfilter and directing the
@@ -202,15 +200,7 @@ class Engine(cxaiebase.AIEScheduler):
         things go wrong, not to cause things to go wrong.
         """
         self.state['environ'] = env = os.environ.copy()
-        if logfile or 'CX_LOG' not in env:
-            if logfile is None:
-                # The bottle may not exist yet, so put the temporary log file in the
-                # desktopdata directory. This way it won't be too much of a bother if
-                # we are unable to move it to the bottle later.
-                logdir = os.path.join(cxproduct.get_user_dir(), "desktopdata")
-                logfile = os.path.join(logdir, "%s.%d.log" % (self.installtask.profile.appid, os.getpid()))
-                self.state['tmplogfile'] = True
-
+        if logfile:
             if not cxutils.mkdirs(os.path.dirname(logfile)):
                 # We'll just install without logging
                 return
@@ -219,7 +209,7 @@ class Engine(cxaiebase.AIEScheduler):
                 f = None
 
                 try:
-                    f = open(logfile, "w")
+                    f = open(logfile, "w", encoding='utf8', errors='surrogateescape') # pylint: disable=R1732
                 except IOError as ioe:
                     cxlog.log("unable to open log file: " + ioe.strerror)
                 else:
@@ -242,7 +232,7 @@ class Engine(cxaiebase.AIEScheduler):
                 self.state['logfile'] = logfile
                 self.state['logfifo'] = logfifo
                 try:
-                    f = self.state['logfifo_file'] = open(logfifo, "w")
+                    f = self.state['logfifo_file'] = open(logfifo, "w", encoding='ascii') # pylint: disable=R1732
                 except IOError as ioe:
                     cxlog.log("unable to open the fifo: " + ioe.strerror)
                 else:
@@ -250,11 +240,17 @@ class Engine(cxaiebase.AIEScheduler):
                     f.write('\n')
                     f.flush()
 
-        if channels is not None:
-            env['CX_DEBUGMSG'] = channels
-        elif 'CX_DEBUGMSG' not in env:
-            env['CX_DEBUGMSG'] = "+cxreboot,-cxassoc,-cxmenu"
+        if 'CX_LOG' in env:
+            if channels is not None:
+                env['CX_DEBUGMSG'] = channels
+            elif 'CX_DEBUGMSG' not in env:
+                env['CX_DEBUGMSG'] = "+cxreboot,-cxassoc,-cxmenu"
 
+            if extra_env:
+                env['CX_ENV'] = extra_env
+        else:
+            # Hack for bug 20989
+            env['CX_LOG'] = '/dev/null'
 
     @cxobjc.python_method
     def stop_logging(self):
@@ -277,7 +273,7 @@ class Engine(cxaiebase.AIEScheduler):
         # going away) so the log file is not empty when we show the log to
         # the user. But don't wait too long in case cxlogfilter failed
         # somehow.
-        for _dummy in cxutils.range(5):
+        for _dummy in range(5):
             if not os.path.exists(logfifo):
                 break
             time.sleep(0.2)
@@ -289,28 +285,22 @@ class Engine(cxaiebase.AIEScheduler):
             pass
         del self.state['logfifo']
 
-        if 'tmplogfile' in self.state:
-            # Move the log file to its final resting place
-            logfile = self.state['logfile']
-            bottlename = self.installtask.bottlename
-            # If the bottle is messed up, expand_win_string can throw an exception
-            try:
-                dstdir = self.expand_win_string("%temp%")
-            except IOError:
-                dstdir = ""
-            if dstdir == "":
-                # The bottle creation probably failed or the bottle is messed up
-                return
-
-            dstdir = bottlequery.get_native_path(bottlename, dstdir)
-            # FIXME: Build a unique filename for the log file
-            dst = cxutils.string_to_str(os.path.join(dstdir, cxutils.sanitize_bottlename(cxlog.to_str(self.installtask.profile.name)) + ".log"))
-            try:
-                shutil.move(logfile, dst)
-                self.state['logfile'] = dst
-            except IOError as ioe:
-                cxlog.log("unable to move %s to %s: %s" % (cxlog.debug_str(logfile), cxlog.debug_str(dst), ioe.strerror))
-
+    @cxobjc.python_method
+    def log_system_info(self, system_info):
+        logfifo_file = self.state.get('logfifo_file')
+        if logfifo_file:
+            logfifo_file.write(system_info)
+            logfifo_file.flush()
+        else:
+            logfile = self.state.get('logfile')
+            if logfile:
+                try:
+                    f = open(logfile, 'a', encoding='utf8') # pylint: disable=R1732
+                except IOError as ioe:
+                    cxlog.log("unable to open log file: " + ioe.strerror)
+                else:
+                    f.write(system_info)
+                    f.close()
 
     @cxobjc.python_method
     def _getlogfile(self):
@@ -373,7 +363,7 @@ class Engine(cxaiebase.AIEScheduler):
                 self._scan.append(aiecore)
                 children_scan.add(aiecore)
         elif app_profile.steamid and \
-            u"com.codeweavers.c4.206" in installer.pre_dependencies:
+            "com.codeweavers.c4.206" in installer.pre_dependencies:
             # Dependency to be installed via steam id
             aiecore.state['install_source'] = "steam://install/%s" % app_profile.steamid
             aiecore.state['installer_file'] = aiecore.state['install_source']
@@ -445,7 +435,7 @@ class Engine(cxaiebase.AIEScheduler):
             aiecore.prepare()
 
 
-    def __init__(self, installtask, log_filename=None, log_channels=None):
+    def __init__(self, installtask, log_filename=None, log_channels=None, log_env=None):
         cxaiebase.AIEScheduler.__init__(self)
         self.installtask = installtask
         # Stores the state information used at run time
@@ -493,7 +483,7 @@ class Engine(cxaiebase.AIEScheduler):
         #      actually run (and thus does not ask the user to insert the
         #      already present CD), but does not prevent other tasks from
         #      running.
-        install_source = installtask.GetInstallerSource()
+        install_source = installtask.installerSource
         if install_source and os.path.isdir(install_source):
             self._dummy_app = cxaiecore.AIECore(self, profiles.unknown_installer().copy())
             self._dummy_app.create_media_tasks()
@@ -593,11 +583,11 @@ class Engine(cxaiebase.AIEScheduler):
 
         # 8. Make the PostInstall task depend on all AIECore objects.
         post_install = cxaiemisc.get_postinstall_task(self)
-        for aiecore in cxutils.itervalues(self._aiecores):
+        for aiecore in self._aiecores.values():
             post_install.add_dependency(aiecore)
 
         # 9. Initialize logging
-        self.start_logging(log_filename, log_channels)
+        self.start_logging(log_filename, log_channels, log_env)
 
         # On the Mac, prevent cycling the MIDI server, since launchd throttles
         # the rate at which it can be restarted.
@@ -619,10 +609,10 @@ class Engine(cxaiebase.AIEScheduler):
 
     # Special initializer for objc. This must always be called explicitly
     # on the Mac.
-    def initWithInstallTask_logFile_channels_(self, installtask, log_filename, log_channels):
+    def initWithInstallTask_logFile_channels_env_(self, installtask, log_filename, log_channels, log_env):
         self = cxobjc.Proxy.nsobject_init(self)
         if self is not None:
-            self.__init__(installtask, log_filename, log_channels)
+            self.__init__(installtask, log_filename, log_channels, log_env)
         return self
 
 
