@@ -286,15 +286,7 @@ class Section(MutableMapping):
         field names may not match the case they have in the file.
         """
         # This method is optional and is provided for efficiency
-        return cxutils.iterkeys(self._values)
-
-    def iteritems(self):
-        """Returns an iterator over the section's field name and value pairs.
-
-        Note that only uncommented fields are returned. Also the case of the
-        field names may not match the case they have in the file.
-        """
-        return cxutils.iteritems(self._values)
+        return iter(self._values.keys())
 
     def keys(self):
         """Returns a list containing the section's field names.
@@ -395,7 +387,7 @@ class Raw(MutableMapping, cxobservable.Object):
         lock) so calling this method from outside cxconfig should rarely be
         needed.
         """
-        self._obj_lock.acquire()
+        self._obj_lock.acquire() # pylint: disable=R1732
 
     def unlock(self):
         """Unlocks the object. See lock() for details."""
@@ -450,13 +442,10 @@ class Raw(MutableMapping, cxobservable.Object):
         """
         if self.read_only:
             raise ValueError('This object is currently read-only')
-        thisfile = open(filename, 'r')
-        lines = list(thisfile.readlines())
-        thisfile.close()
+        with open(filename, 'r', encoding='utf8') as thisfile:
+            lines = list(thisfile.readlines())
 
-        if lines and lines[0].startswith('\xef\xbb\xbf'): #utf8 bom(b)
-            lines[0] = lines[0][3:]
-        if lines and lines[0].startswith(u'\ufeff'):
+        if lines and lines[0].startswith('\ufeff'):
             lines[0] = lines[0][1:]
         self.read_lines(lines)
 
@@ -494,7 +483,7 @@ class Raw(MutableMapping, cxobservable.Object):
             if dirname and not os.path.exists(dirname):
                 os.makedirs(dirname)
             temp_file = out + '.tmp'
-            file_obj = open(temp_file, 'wb')
+            file_obj = open(temp_file, 'wb') # pylint: disable=R1732
 
         self.lock()
         try:
@@ -584,19 +573,7 @@ class Raw(MutableMapping, cxobservable.Object):
         Since the section names are case insensitive this should not bother
         the caller or he should be able to deal with case differences anyway."""
         # This method is optional and is provided for efficiency
-        return cxutils.iterkeys(self._real_sections)
-
-    def iteritems(self):
-        """Returns an iterator over the configuration's section name and object
-        pairs.
-
-        Note that because section names are case insensitive, the case of the
-        section names returned by the iterator may not match the case they have
-        in the file (to get the unaltered section name, query the section's
-        name attribute).
-        """
-        # This method is optional and is provided for efficiency
-        return cxutils.iteritems(self._real_sections)
+        return iter(self._real_sections.keys())
 
     def keys(self):
         """Returns a list containing the configuration's section names.
@@ -936,8 +913,6 @@ class _StackSection(Mapping):
                         seen.add(fieldname)
                         yield fieldname
 
-    # Let Mapping figure out how to provide iteritems()
-
     def keys(self):
         """See Section.keys() for details."""
         fields = set()
@@ -1092,8 +1067,6 @@ class Stack(Mapping, cxobservable.Object):
                     seen.add(name)
                     yield name
 
-    # Let Mapping figure out how to provide iteritems()
-
     def keys(self):
         """See Raw.keys() for details."""
         names = set()
