@@ -1,5 +1,6 @@
 # (c) Copyright 2014-2021. CodeWeavers, Inc.
 
+import collections
 import os.path
 import re
 
@@ -42,7 +43,7 @@ def run_from_main_thread(fun):
     gtk_level = Gtk.main_level()
     if gtk_level != 0:
         _LOCK = threading.Lock()
-        _LOCK.acquire()
+        _LOCK.acquire() # pylint: disable=R1732
 
     Gdk.threads_add_idle(0, fun, None)
 
@@ -208,11 +209,7 @@ def refused(gui, title, text, prompt):
         print(title)
         print(text)
         try:
-            # Python 3's input() is Python 2's raw_input()!
-            try:
-                r = raw_input(cxutils.string_to_utf8(prompt) + " [Y/n] ")
-            except NameError:
-                r = input(prompt + " [Y/n] ")
+            r = input(prompt + " [Y/n] ")
             retcode = 0 if r in ('', 'y', 'Y') else 1
         except EOFError:
             retcode = -2
@@ -329,7 +326,7 @@ def detect_distribution():
 
 
         best_priority = -1
-        for _filename, distroid in glob_tree.matches(cxutils.b('/')):
+        for _filename, distroid in glob_tree.matches(b'/'):
             if _DISTROID is None or \
                     best_priority < CXFIXES.distributions[distroid].priority:
                 _DISTROID = distroid
@@ -390,7 +387,9 @@ def debian_add_architecture(arch, cmd):
 
 _ERRORS = {}
 
-def add_error(errid, title=None, level=None):
+ErrorInfo = collections.namedtuple('ErrorInfo', ('level', 'title', 'lib'))
+
+def add_error(errid, title=None, level=None, lib=None):
     _load_fixes()
     fix = CXFIXES.fixes.get(errid, None)
     if level is None:
@@ -401,7 +400,13 @@ def add_error(errid, title=None, level=None):
             title = fix.title
         if title is None:
             title = _('Unknown error %s') % errid
-    _ERRORS[errid] = (level, title)
+    if fix is None:
+        pass
+    elif lib is None:
+        lib = fix.lib
+    elif fix.lib is not None and lib != fix.lib:
+        cxlog.warn("The %s fix provides %s, not the needed %s" % (errid, fix.lib, lib))
+    _ERRORS[errid] = ErrorInfo(level, title, lib)
 
 def add_masked_errors(levels=None):
     """Check if there are any other errors that these may have masked by the
@@ -446,7 +451,7 @@ def get_error_description(errid):
 
 def get_error_title(errid):
     err = _ERRORS.get(errid, None)
-    return errid if err is None else err[1]
+    return errid if err is None else err.title
 
 def get_error_url(errid):
     # Redirect the BrokenLib issues to the MissingLib ones:
@@ -491,7 +496,7 @@ def get_fix(errid, distroid=None, bitness=None):
         fallbackid = errid
         while fallbackid:
             distfixes = CXFIXES.fixes[fallbackid].distfixes
-            for key in cxutils.keys(distfixes):
+            for key in distfixes:
                 if key not in merged_fixes:
                     merged_fixes[key] = distfixes[key]
             fallbackid = CXFIXES.fixes[fallbackid].fallback
@@ -513,12 +518,6 @@ def get_fix(errid, distroid=None, bitness=None):
 
     return (merged_fixes, None, None)
 
-def has_fix(errid):
-    (distfixes, fix_distroid, fix_bitness) = get_fix(errid)
-    if fix_distroid and distfixes[(fix_distroid, fix_bitness)].packages:
-        return True
-    return False
-
 def get_distribution_property(distroid, name):
     """Returns the distribution's property, using the fallbacks if necessary.
     If there is no such property, then None is returned.
@@ -532,14 +531,19 @@ def get_distribution_property(distroid, name):
         dist = CXFIXES.distributions[dist.fallback]
     return dist.__dict__[name]
 
-def get_packages(errid, distroid=None, bitness=None):
+def get_packages(errid, distroid=None, bitness=None, pre=None):
     _load_fixes()
 
     # Return the packages for all the relevant errors
     (distfixes, fix_distroid, fix_bitness) = get_fix(errid, distroid, bitness)
-    if fix_distroid:
-        return distfixes[(fix_distroid, fix_bitness)].packages
-    return []
+    if fix_distroid is None:
+        return []
+    packages = []
+    if pre or pre is None:
+        packages = list(distfixes[(fix_distroid, fix_bitness)].prepackages)
+    if not pre or pre is None:
+        packages.extend(distfixes[(fix_distroid, fix_bitness)].packages)
+    return packages
 
 def get_package_key(name):
     if name.endswith(':i386'): # Debian
@@ -584,8 +588,10 @@ def get_fix_command(update=True, checklock=False, allatonce=True, yes=False):
     package_sets = {}
     has_arch32 = False
     has_deb32 = False
+    prepackages = set()
     for errid in _ERRORS:
-        fix_packages = get_packages(errid, distroid, bitness)
+        prepackages.update(get_packages(errid, distroid, bitness, pre=True))
+        fix_packages = get_packages(errid, distroid, bitness, pre=False)
         if fix_packages:
             fixable_errids.append(errid)
             package_set = set(fix_packages)
@@ -615,6 +621,8 @@ def get_fix_command(update=True, checklock=False, allatonce=True, yes=False):
     if not package_sets:
         return ([], None)
 
+    if prepackages:
+        cmd = pkg_cmd + ' ' + cxutils.argvtocmdline(prepackages) + '; ' + cmd
     if allatonce:
         all_packages = set()
         for package_set in package_sets.values():
@@ -748,9 +756,13 @@ def markup_error_message(message):
     return '\n'.join(lines)
 
 def get_fix_message(distfixes, distroid, bitness, distname=True):
-    args = [get_distribution_property(detect_distribution(), 'packagecmd')]
-    packages = distfixes[(distroid, bitness)].packages
-    args.extend(sorted(packages))
+    cmd = get_distribution_property(detect_distribution(), 'packagecmd')
+    args = [cmd]
+    prepackages = distfixes[(distroid, bitness)].prepackages
+    if prepackages:
+        args.extend(sorted(prepackages))
+        args.extend((';', cmd))
+    args.extend(sorted(distfixes[(distroid, bitness)].packages))
     msg = ' '.join(args)
 
     if distname:
@@ -791,7 +803,7 @@ def get_error_message(prefix=True, verbose=False):
         if verbose and distfixes:
             # Show the fixes for other platforms
             lines = []
-            for (other_distroid, other_bitness) in cxutils.keys(distfixes):
+            for (other_distroid, other_bitness) in distfixes:
                 if (other_distroid != distroid or \
                     (other_bitness is not None and other_bitness != bitness)) and \
                     distfixes[(other_distroid, other_bitness)].packages:
@@ -834,8 +846,18 @@ def get_error_message(prefix=True, verbose=False):
 
             msg = errid2msg[seed_errid]
             for errid in sorted(msg2errid[(group, msg)]):
-                bits = '64' if errid.endswith('.amd64') else '32'
-                message += '* %s (%s-bit)\n' % (get_error_url(errid), bits)
+                lib = _ERRORS[errid].lib
+                if errid.endswith('.amd64'):
+                    bits = '64-bit, '
+                elif lib is not None or errid + '.amd64' in CXFIXES.fixes:
+                    bits = '32-bit, '
+                else:
+                    bits = ''
+                lib = '' if lib is None else '%s, ' % lib
+                level = _ERRORS[errid].level
+                if level is None:
+                    level = _('obsolete')
+                message += '* %s (%s%s%s)\n' % (get_error_url(errid), lib, bits, level)
                 added.add(errid)
             if msg:
                 message += msg + '\n'
