@@ -1,6 +1,5 @@
 # (c) Copyright 2014-2015. CodeWeavers, Inc.
 
-import sys
 import time
 import traceback
 import xml.sax
@@ -417,16 +416,14 @@ class _CXFixesHandler(xml.sax.handler.ContentHandler):
         try:
             distro.validate()
             self.cxfixes.distributions[distro.distid] = distro
-        except AttributeError:
-            exception = sys.exc_info()[1]
+        except AttributeError as exception:
             self.parse_errors.append(str(exception))
             msg = "the %s distribution is not valid, ignoring it" % distro.distid
             self.parse_errors.append(msg)
             if cxlog.is_on('cxfixesparser'):
                 traceback.print_exc()
                 cxlog.warn(msg)
-        except: # pylint: disable=W0702
-            exception = sys.exc_info()[1] # Python 2 and 3 compatible
+        except Exception as exception: # pylint: disable=W0702
             self.parse_errors.append(str(exception))
             msg = "an unexpected error occurred while parsing the %s distribution, ignoring it" % distro.distid
             self.parse_errors.append(msg)
@@ -480,16 +477,14 @@ class _CXFixesHandler(xml.sax.handler.ContentHandler):
         try:
             cxproduct.validate()
             self.cxfixes.cxproducts[cxproduct.cxid] = cxproduct
-        except AttributeError:
-            exception = sys.exc_info()[1]
+        except AttributeError as exception:
             self.parse_errors.append(str(exception))
             msg = "the %s cxproduct is not valid, ignoring it" % cxproduct.cxid
             self.parse_errors.append(msg)
             if cxlog.is_on('cxfixesparser'):
                 traceback.print_exc()
                 cxlog.warn(msg)
-        except: # pylint: disable=W0702
-            exception = sys.exc_info()[1] # Python 2 and 3 compatible
+        except Exception as exception: # pylint: disable=W0702
             self.parse_errors.append(str(exception))
             msg = "an unexpected error occurred while parsing the %s cxproduct, ignoring it" % cxproduct.cxid
             self.parse_errors.append(msg)
@@ -576,16 +571,14 @@ class _CXFixesHandler(xml.sax.handler.ContentHandler):
         try:
             fix.validate()
             self.cxfixes.fixes[fix.errid] = fix
-        except AttributeError:
-            exception = sys.exc_info()[1]
+        except AttributeError as exception:
             self.parse_errors.append(str(exception))
             msg = "the %s fix is not valid, ignoring it" % fix.errid
             self.parse_errors.append(msg)
             if cxlog.is_on('cxfixesparser'):
                 traceback.print_exc()
                 cxlog.warn(msg)
-        except: # pylint: disable=W0702
-            exception = sys.exc_info()[1] # Python 2 and 3 compatible
+        except Exception as exception: # pylint: disable=W0702
             self.parse_errors.append(str(exception))
             msg = "an unexpected error occurred while parsing the %s fix, ignoring it" % fix.errid
             self.parse_errors.append(msg)
@@ -611,7 +604,8 @@ class _CXFixesHandler(xml.sax.handler.ContentHandler):
 
     def _childof_fixfor(self, name, attrs):
         if name == 'package':
-            self._attr2stringset('packages', 'name', attrs)
+            setname = 'prepackages' if attrs.get('preinstall', False) else 'packages'
+            self._attr2stringset(setname, 'name', attrs)
             self._startof_childless(name)
         else:
             self._startof_skip(name)
@@ -636,10 +630,9 @@ def read_file(filename):
             cxlog.log("parsing '%s' took %0.3fs" % (cxlog.to_str(filename), time.time() - start))
             return (handler.cxfixes, handler.parse_errors)
         handler.parse_errors.extend(errors)
-    except: # pylint: disable=W0702
-        exception = sys.exc_info()[1] # Python 2 and 3 compatible
+    except Exception as exception: # pylint: disable=W0702
         import errno
-        if hasattr(exception, 'errno') and exception.errno == errno.ENOENT:
+        if hasattr(exception, 'errno') and exception.errno == errno.ENOENT: # pylint: disable=E1101
             handler.parse_errors.append("'%s' does not exist" % filename)
         else:
             msg = "an unexpected error occurred while parsing '%s'. Ignoring this file" % filename
