@@ -1,13 +1,9 @@
 # (c) Copyright 2009-2015. CodeWeavers, Inc.
 
-import ctypes
 import os
 import sys
 import traceback
-if sys.version_info < (3,):
-    import urllib
-else:
-    import urllib.parse as urllib # pylint: disable=E0611,E0401
+import urllib.parse as urllib
 
 from gi.repository import GLib
 from gi.repository import GdkPixbuf
@@ -43,7 +39,7 @@ def get_ui_path(basename):
     return os.path.join(ui_dir, basename + ".ui")
 
 
-ICONS = dict()
+ICONS = {}
 def get_std_icon(basename, sizes=cxutils.S_MEDIUM):
     # Assume scanning the cache is faster than disk accesses
     for size in sizes:
@@ -73,53 +69,43 @@ def get_std_icon_list(basename='crossover'):
     icons.sort(key=lambda x: x.get_property('width'))
     return icons
 
+
 def set_default_icon(basename='crossover'):
     Gtk.Window.set_default_icon_list(get_std_icon_list(basename))
 
-def load_icon(names, size, widget, std_icon=None, std_sizes=None, std_recolor=False):
-    "load the first available icon, or return None if none are available"
+
+def get_icon_name(names, size, symbolic=False):
+    """Return the name of the first icon available, None if none are available"""
+
     icon_theme = Gtk.IconTheme.get_default()
-    fgcolor = widget.get_style_context().get_color(Gtk.StateFlags.NORMAL)
     if isinstance(size, Gtk.IconSize):
         size = Gtk.IconSize.lookup(size)[1]
+
+    name_extensions = ['', '-symbolic']
+    if symbolic:
+        name_extensions = ['-symbolic', '']
+
     for name in names:
-        try:
-            result = icon_theme.load_icon(name, size, Gtk.IconLookupFlags.USE_BUILTIN)
-        except: # pylint: disable=W0702
-            pass
-        else:
-            return result
-        try:
-            result = icon_theme.load_icon(name+'-symbolic', size, Gtk.IconLookupFlags.USE_BUILTIN)
-        except: # pylint: disable=W0702
-            pass
-        else:
-            result = result.copy()
-            result = recolor_pixbuf(result, fgcolor, symbolic=True)
-            return result
-    if std_icon:
-        result = get_std_icon(std_icon, std_sizes)
-        if result:
-            result = result.scale_simple(size, size, GdkPixbuf.InterpType.BILINEAR)
-            if std_recolor:
-                result = recolor_pixbuf(result, fgcolor, symbolic=False)
-            return result
+        for name_extension in name_extensions:
+            result = icon_theme.lookup_icon(name + name_extension, size, Gtk.IconLookupFlags.USE_BUILTIN)
+            if result:
+                return name + name_extension
+
     return None
 
-def draw_widget_background(_widget, event, pixbuf):
-    event.window.draw_pixbuf(None, pixbuf, 0, 0, 0, 0)
-    return False
 
+def load_icon(names, size, symbolic=False):
+    "load the first available icon, or return None if none are available"
 
-def set_widget_background(widget, filename):
-    filename = os.path.join(cxutils.CX_ROOT, 'share', 'images', filename)
-    try:
-        pixbuf = GdkPixbuf.Pixbuf.new_from_file(filename)
-    except GLib.Error: # pylint: disable=E0712
-        cxlog.warn("couldn't load icon file %s:\n%s" % (cxlog.debug_str(filename), traceback.format_exc()))
-        return
+    icon_theme = Gtk.IconTheme.get_default()
+    if isinstance(size, Gtk.IconSize):
+        size = Gtk.IconSize.lookup(size)[1]
 
-    widget.connect('expose-event', draw_widget_background, pixbuf)
+    name = get_icon_name(names, size, symbolic)
+    if name:
+        return icon_theme.load_icon(name, size, Gtk.IconLookupFlags.USE_BUILTIN)
+
+    return None
 
 
 # Menu functions
@@ -200,58 +186,6 @@ else:
         _popup_at_screen_rect(menu, rect, Gdk.Gravity.NORTH_WEST, Gdk.Gravity.NORTH_WEST, event)
 
 
-# pylint: disable=C0103
-def blend_colors(a, b, a_weight, b_weight):
-    if isinstance(a, Gdk.Color) and isinstance(b, Gdk.Color):
-        total_weight = a_weight + b_weight
-        return Gdk.Color(
-            (a.red * a_weight + b.red * b_weight) // total_weight,
-            (a.green * a_weight + b.green * b_weight) // total_weight,
-            (a.blue * a_weight + b.blue * b_weight) // total_weight)
-    if isinstance(a, Gdk.RGBA) and isinstance(b, Gdk.RGBA):
-        total_weight = a_weight + b_weight
-        return Gdk.RGBA(
-            (a.red * a_weight + b.red * b_weight) / total_weight,
-            (a.green * a_weight + b.green * b_weight) / total_weight,
-            (a.blue * a_weight + b.blue * b_weight) / total_weight,
-            (a.alpha * a_weight + b.alpha * b_weight) / total_weight)
-    raise TypeError("invalid types for a and b")
-
-def recolor_pixbuf(pixbuf, color, symbolic=False):
-    width = pixbuf.get_property('width')
-    height = pixbuf.get_property('height')
-    result = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, width, height)
-    pixbuf.copy_area(0, 0, width, height, result, 0, 0)
-
-    pixels = ctypes.cast(ctypes.c_void_p(result.get_property('pixels')), ctypes.POINTER(ctypes.c_ubyte))
-
-    if isinstance(color, Gdk.RGBA):
-        r = int(color.red * 255 + 0.5)
-        g = int(color.green * 255 + 0.5)
-        b = int(color.blue * 255 + 0.5)
-        a = color.alpha
-    elif isinstance(color, Gdk.Color):
-        r = int(color.red / 256)
-        g = int(color.green / 256)
-        b = int(color.blue / 256)
-        a = 1.0
-    else:
-        raise TypeError("invalid type for color")
-
-    stride = result.get_property('rowstride')
-
-    for y in range(height):
-        i = y * stride
-        for _x in range(width):
-            if not symbolic or (0xbe == pixels[i] == pixels[i+1] == pixels[i+2]):
-                pixels[i] = r
-                pixels[i+1] = g
-                pixels[i+2] = b
-                pixels[i+3] = int(pixels[i+3]*a+0.5)
-            i += 4
-
-    return result
-
 def rgba_parse(spec):
     result = Gdk.RGBA()
     if not result.parse(spec):
@@ -281,7 +215,7 @@ def CXMessageDlg(message=None, primary=None, secondary=None, markup=None, button
         if message is None:
             text = []
             if primary is not None:
-                text.append(u'<span weight="bold" size="larger">%s</span>' % cxutils.html_escape(primary))
+                text.append('<span weight="bold" size="larger">%s</span>' % cxutils.html_escape(primary))
             if secondary is not None:
                 text.append(cxutils.html_escape(secondary))
             markup = '\n\n'.join(text)
@@ -291,14 +225,18 @@ def CXMessageDlg(message=None, primary=None, secondary=None, markup=None, button
         # &nbsp; is an important HTML typographic entity (e.g. in French) but
         # it is not supported by GTK+. So manually convert it to the
         # corresponding Unicode character
-        markup = markup.replace(u"&nbsp;", u"\u00a0")
+        markup = markup.replace("&nbsp;", "\u00a0")
     if message_type is None:
         cxlog.warn("Callers to cxguitools.CXMessageDlg should always set an alert type.")
         message_type = Gtk.MessageType.INFO
     if button_array:
         dialog = Gtk.MessageDialog(buttons=Gtk.ButtonsType.NONE, modal=True, transient_for=parent, message_type=message_type)
         for button in button_array:
-            dialog.add_button(button[0], button[1])
+            new_button = dialog.add_button(button[0], button[1])
+            try:
+                new_button.get_style_context().add_class(button[2])
+            except Exception:
+                pass
     else:
         dialog = Gtk.MessageDialog(buttons=buttons, modal=True, transient_for=parent, message_type=message_type)
     dialog.set_markup(markup)
@@ -322,7 +260,7 @@ def show_malware_dialog(c4pfile, parent=None, stop_loop=False):
     primary = _("The file '%(filename)s' contains malware") % {'filename': c4pfile.filename}
     secondary = _("This file cannot be used because it would install malicious software or otherwise harm your computer.")
     CXMessageDlg(secondary=secondary, primary=primary,
-                 button_array=[[_(u"More Info\u2026"), 0],
+                 button_array=[[_("More Info…"), 0],
                                [Gtk.STOCK_CLOSE, 1]],
                  user_data=(c4pfile, stop_loop),
                  response_function=_malware_response,
@@ -351,7 +289,7 @@ def warn_if_root():
         warning = _("Running this tool as root is very dangerous, and will only allow you to configure CrossOver for the root user.")
         if distversion.HAS_MULTI_USER:
             warning = _("%s\n\nIf you wish to create a bottle that all the users of this computer can access, log in as a user and use the 'Publish Bottle' feature.") % warning
-        CXMessageDlg(warning, buttons=None, button_array=[[_("Exit"), 1], [_("Configure CrossOver for Root"), 0]], response_function=return_from_root_check, message_type=Gtk.MessageType.WARNING)
+        CXMessageDlg(warning, buttons=None, button_array=[[_("Exit"), 1, 'suggested-action'], [_("Configure CrossOver for Root"), 0]], response_function=return_from_root_check, message_type=Gtk.MessageType.WARNING)
         Gtk.main()
 
 
@@ -382,7 +320,7 @@ def add_filters(file_picker, filters):
     file_filters = {}
     default = ''
 
-    if 'archives' in  filters:
+    if 'archives' in filters:
         file_filter = Gtk.FileFilter()
         # These are Unix files so keep them case-sensitive
         file_filter.add_pattern("*.cpio.7z")
@@ -396,7 +334,7 @@ def add_filters(file_picker, filters):
         file_filter.add_pattern("*.tgz")
         file_filters[_("Unix Archives")] = file_filter
 
-    if 'cxarchives' in  filters:
+    if 'cxarchives' in filters:
         file_filter = Gtk.FileFilter()
         # These are Unix files so keep them case-sensitive
         file_filter.add_pattern("*.cxarchive")
@@ -454,8 +392,7 @@ def add_filters(file_picker, filters):
         add_ipattern(file_filter, "*")
         file_filters[_("All Files")] = file_filter
 
-    names = cxutils.keys(file_filters)
-    names.sort()
+    names = sorted(file_filters)
     for name in names:
         file_filter = file_filters[name]
         file_filter.set_name(name)
