@@ -1,21 +1,14 @@
 # (c) Copyright 2020-2021 CodeWeavers, Inc.
 
 import os
-import sys
 import json
 import shutil
 import tempfile
 
-if sys.version_info >= (3,):
-    import urllib.request as urllib_request # pylint: disable=E0611,E0401
-    import urllib.error as urllib_error # pylint: disable=E0401,E0611
-    import urllib.parse as urlparse # pylint: disable=E0401,E0611
-    from urllib.parse import urlencode # pylint: disable=E0401,E0611
-else:
-    import urllib2 as urllib_request # pylint: disable=E0401
-    import urllib2 as urllib_error # pylint: disable=E0401
-    from urllib import urlencode # pylint: disable=E0611
-    import urlparse # pylint: disable=E0401
+import urllib.request as urllib_request
+import urllib.error as urllib_error
+import urllib.parse as urlparse
+from urllib.parse import urlencode
 
 from gi.repository import GLib
 from gi.repository import Gtk
@@ -37,7 +30,7 @@ import webtoken
 # for localization
 from cxutils import cxgettext as _
 
-class UpdateController(object):
+class UpdateController:
 
     def __init__(self, parent=None, is_reminder=False):
         self.xml = Gtk.Builder()
@@ -158,7 +151,7 @@ class UpdateController(object):
         self.xml.get_object("CancelButton").show()
         self.xml.get_object("UpdateProgress").show()
 
-        self.xml.get_object("UpdateLabel").set_text(_(u'Downloading\u2026'))
+        self.xml.get_object("UpdateLabel").set_text(_('Downloading…'))
 
         pyop.sharedOperationQueue.enqueue(DownloadOperation(self.url, self))
 
@@ -167,14 +160,14 @@ class UpdateController(object):
             self.destroy()
 
         if error or not installer_file:
-            self.xml.get_object("UpdateLabel").set_text(_(u'An error occurred while downloading the update.'))
+            self.xml.get_object("UpdateLabel").set_text(_('An error occurred while downloading the update.'))
             self.xml.get_object("UpdateProgress").hide()
             self.xml.get_object("CancelButton").hide()
             self.xml.get_object("OkButton").show()
             return
 
         self.xml.get_object("CancelButton").set_sensitive(False)
-        self.xml.get_object("UpdateLabel").set_text(_(u'Installing\u2026'))
+        self.xml.get_object("UpdateLabel").set_text(_('Installing…'))
 
         self.pulse = True
 
@@ -200,7 +193,7 @@ class UpdateController(object):
             self.destroy()
 
         if error:
-            self.xml.get_object("UpdateLabel").set_text(_(u'An error occurred while installing the update.'))
+            self.xml.get_object("UpdateLabel").set_text(_('An error occurred while installing the update.'))
             self.xml.get_object("UpdateProgress").hide()
             self.xml.get_object("CancelButton").hide()
             self.xml.get_object("OkButton").show()
@@ -351,7 +344,7 @@ def get_version():
     '''Put the version number in the right format for the REST API'''
     return '.'.join(distversion.CX_VERSION.split('.')[0:3])
 
-class UpdateUtils(object):
+class UpdateUtils:
 
     def __init__(self):
         self.login_required = False
@@ -374,7 +367,9 @@ class UpdateUtils(object):
             header = {'User-Agent': cxurlget.USER_AGENT}
             request = urllib_request.Request(url, None, header)
 
-            response = urllib_request.urlopen(request, timeout=5)
+            # pylint suggests using 'with' statement, but context manager functionality
+            # on the response object is undocumented
+            response = urllib_request.urlopen(request, timeout=5) #pylint: disable=R1732
             data = json.loads(cxutils.string_to_unicode(response.read()))
 
             cxlog.log('Obtained download data:')
@@ -446,7 +441,7 @@ def get_cached_installer(url):
     return None
 
 
-class Downloader(object):
+class Downloader:
 
     def __init__(self, url, controller):
         self._getter = None
