@@ -1,15 +1,15 @@
 # (c) Copyright 2008, 2010-2012. CodeWeavers, Inc.
-package CXBT_win8_64;
+package CXBT_win10_64;
 use strict;
 
 use CXLog;
 use CXUtils;
 
 my $productid=CXUtils::get_product_id();
-my $template="win8_64";
+my $template="win10_64";
 my $datadir="$ENV{CX_ROOT}/share/crossover/bottle_data";
 my $templatedir="$ENV{CX_ROOT}/share/crossover/bottle_templates/$template";
-my $windows_version="win8";
+my $windows_version="win10";
 my $winearch="win64";
 
 
@@ -220,7 +220,6 @@ sub create($$$)
     $s->set("MenuStrip", $menu_strip) if (defined $menu_strip);
     $s->set("MenuMode", "ignore");
     $s->set("AssocMode", "ignore");
-    $s->set("NSPluginMode", "ignore");
     $s->set("WineArch", $winearch);
     $cxbottle->write("$ENV{WINEPREFIX}/cxbottle.conf");
 
@@ -762,7 +761,6 @@ sub upgrade($$$)
 
     my $uassoc="";
     my $umenu="";
-    my $unsplugin="";
     if ($old_version lt "05.0")
     {
         # If the same user who owns the bottle installed CrossOver,
@@ -777,7 +775,7 @@ sub upgrade($$$)
         }
 
         # Recreate the menus, associations and plugins from scratch
-        $uassoc=$umenu=$unsplugin="sync";
+        $uassoc=$umenu="sync";
     }
     if ($old_version lt "10.0")
     {
