@@ -220,7 +220,6 @@ sub create($$$)
     $s->set("MenuStrip", $menu_strip) if (defined $menu_strip);
     $s->set("MenuMode", "ignore");
     $s->set("AssocMode", "ignore");
-    $s->set("NSPluginMode", "ignore");
     $s->set("WineArch", $winearch);
     $cxbottle->write("$ENV{WINEPREFIX}/cxbottle.conf");
 
@@ -762,7 +761,6 @@ sub upgrade($$$)
 
     my $uassoc="";
     my $umenu="";
-    my $unsplugin="";
     if ($old_version lt "05.0")
     {
         # If the same user who owns the bottle installed CrossOver,
@@ -777,7 +775,7 @@ sub upgrade($$$)
         }
 
         # Recreate the menus, associations and plugins from scratch
-        $uassoc=$umenu=$unsplugin="sync";
+        $uassoc=$umenu="sync";
     }
     if ($old_version lt "10.0")
     {
